#include "myitemmodel.h"

MyItemModel::MyItemModel(QObject* parent) : QAbstractItemModel(parent)
{
    root = new TreeItem();
}

MyItemModel::~MyItemModel(){
    delete root;
}



int MyItemModel::rowCount(const QModelIndex& parent) const {
    TreeItem* parentItem = getItem(parent);
    return parentItem->getChildrenCount();
}

int MyItemModel::columnCount(const QModelIndex &parent) const{
    return root->columnCount();
}

QVariant MyItemModel::data(const QModelIndex &index, int role) const{
    if (!index.isValid()){
        return QVariant();
    }
    if (role != Qt::DisplayRole){
        return QVariant();
    }
    TreeItem* item = static_cast<TreeItem*>(index.internalPointer());
    return item->data(index.column());
}


QVariant MyItemModel::headerData(int section, Qt::Orientation orientation, int role) const{
    //Im tutorial stand mehr. erstmal ignorieren.
    return root->data(section);
}

QModelIndex MyItemModel::index(int row, int column, const QModelIndex& parent) const{
    if (!hasIndex(row, column, parent)){
        return QModelIndex();
    }
    TreeItem* parentItem;

    if (!parent.isValid()){
        parentItem = root;
    } else{
        parentItem = static_cast<TreeItem*>(parent.internalPointer());
    }

    TreeItem* childItem = parentItem->getChild(row);
    if (childItem){
        return createIndex(row, column, childItem);
    }else{
        return QModelIndex();
    }
}

QModelIndex MyItemModel::parent(const QModelIndex &index) const{
    if (!index.isValid()){
        return QModelIndex();
    }

    TreeItem* childItem = static_cast<TreeItem*>(index.internalPointer());
    TreeItem* parentItem = childItem->getParent();
    if (parentItem == root){
        return QModelIndex();
    }else{
        return createIndex(parentItem->childNumber(), 0, parentItem);
    }
}

//enable editing and selection.
Qt::ItemFlags MyItemModel::flags(const QModelIndex &index) const{
    if (!index.isValid()){
        return 0;
    }else{
        return Qt::ItemIsEditable | Qt::ItemIsSelectable | QAbstractItemModel::flags(index);
    }
}

//Helferfunktion
TreeItem* MyItemModel::getItem(const QModelIndex& index) const{
    if (index.isValid()){
        TreeItem* item = static_cast<TreeItem*>(index.internalPointer());
        if (item) {
            return item;
        }
    }
    return root;
}


void MyItemModel::addNewTransformToRoot(TreeItem *rbt, TreeItem* prim){
    beginInsertRows(QModelIndex(), 0, 0);
        qDebug() << "Add child " << rbt->data(0);
        root->addChild(rbt);
    endInsertRows();
    QModelIndex i = index(0,0,QModelIndex());
    beginInsertRows(i,0,0);

    qDebug() << "Add primitive " << prim->data(0) << " at " << i;
    rbt->addChild(prim);
    endInsertRows();
//    return true;
}

void MyItemModel::removeTransformFromRoot(TreeItem *rbt){
    int row = -1;
    for (int i = 0; i < root->getChildren().length(); i++){
        if (root->getChild(i) == rbt){
            row = i;
            break;
        }
    }
    if (row < 0) qDebug("item not found");
    else {
//        QModelIndex i = index(row,0,QModelIndex());
        beginRemoveRows(QModelIndex(),row,row);
        root->removeChild(rbt);
        endRemoveRows();
    }
}
